#define F_CPU 16000000UL
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define set_bit(value, bit) ( _SFR_BYTE(value) |= _BV(bit) )
#define clear_bit(value, bit) ( _SFR_BYTE(value) &= ~_BV(bit) )

volatile uint8_t state2 = 0;     // LED의 상태
volatile uint8_t count = 0;   // 오버플로 인터럽트 발생 횟수

ISR(PCINT1_vect) {      // 핀 변화 인터럽트
  state2 = !state2;       // LED 상태 전환
}

ISR(TIMER0_OVF_vect) {      // 오버플로 인터럽트
  count++;
}

void PCINT1_init(void) {
  PCICR |= (1 << PCIE1);    // 핀 변화 인터럽트 1 허용
  PCMSK1 |= (1 << PCINT9);    // PJ0 핀의 핀 변화 인터럽트 허용
}

void TIMER_COUNTER0_init(void) {
  TCCR0B |= (1 << CS02) | (1 << CS00);  // 분주비를 1024로 설정
  TIMSK0 |= (1 << TOIE0);     // 오버플로 인터럽트 허용
}

int main(void) {
  PCINT1_init();        // INT0 인터럽트 설정
  TIMER_COUNTER0_init();    // 오버플로 인터럽트 설정
  sei();              // 전역적으로 인터럽트 허용

  clear_bit(DDRJ, 0);       // 버튼 연결 핀을 입력으로 설정
  set_bit(DDRE, 4);       // LED 연결 핀 PE4를 출력으로 설정
  set_bit(DDRE, 5);       // LED 연결 핀 PE5를 출력으로 설정
  clear_bit(PORTE, 4);      // PB4 핀에 연결된 LED를 끈 상태로 시작
  clear_bit(PORTE, 5);      // PB5 핀에 연결된 LED를 끈 상태로 시작

  uint8_t state1 = 0;

  while (1) {
    if (count == 64) {      // 오버플로 64회 발생 = 1초 경과
      count = 0;      // 인터럽트 발생 횟수 초기화
      state1 = !state1;       // LED1 상태 반전
      if (state1) set_bit(PORTE, 4);  // LED1 켜기
      else clear_bit(PORTE, 4);     // LED1 끄기
    }

    if (state2) set_bit(PORTE, 5);    // LED2 켜기
    else clear_bit(PORTE, 5);   // LED2 끄기
  }

  return 0;
}
