#define F_CPU 16000000L
#include <avr/io.h>
#include <util/delay.h>
#include "millis2560.h"
#include "UART0.h"

#define set_bit(value, bit) ( _SFR_BYTE(value) |= _BV(bit) )
#define clear_bit(value, bit) ( _SFR_BYTE(value) &= ~_BV(bit) )

int main(void) {
  UART0_init();       // UART 시리얼 통신 초기화
  millis2560_init();      // 타이머/카운터 초기화

  set_bit(DDRB, 7);       // PB7 핀을 출력으로 설정
  clear_bit(PORTB, 7);      // LED는 끈 상태로 시작

  uint8_t state = 0;      // LED 상태
  // 실행 시간
  unsigned long time_p = millis2560(), time_c;
  long loop_count = 0;      // while(1) 루프 실행 횟수

  while (1) {
    time_c = millis2560();      // 현재까지의 실행 시간
    loop_count++;

    if (time_c - time_p >= 1000) {  // 1초 이상이 경과한 경우
      time_p = time_c;      // 시작 시간 변경
      state = !state;       // LED 상태 반전
      if (state) set_bit(PORTB, 7);   // LED 켜기
      else clear_bit(PORTB, 7);     // LED 끄기

      UART0_print(loop_count);    // while(1) 루프 실행 횟수 출력
      UART0_write('\n');
      loop_count = 0;
    }
  }

  return 0;
}
